<?php

class TreeBehavior extends CActiveRecordBehavior
{
	public $id='id';
	public $parent_id='parent_id';
	public $left='lft';
	public $right='rgt';
	public $level='level';
	public $name='name';
	
	public $isAllowedManyRoots=false;
	
	//根据parent_id树，重新建立左右结节树，防止该树发生错误时，备用
	public function rebuildTree($parent_id=0, $left=0) {
		$owner=$this->getOwner();
		
		$right = $left+1;
        // get all children of this node
        $rows=$owner->findAll("$this->parent_id='$parent_id'");
        foreach ($rows as $row) {
        	// recursive execution of this function for each
        	// child of this node
        	// $right is the current right value, which is
        	// incremented by the rebuildTree function
        	$right = $this->rebuildTree($row->getAttribute($this->id), $right);
        }
        // we've got the left value, and now that we've processed
        // the children of this node we also know the right value
        $owner->updateAll(array($this->left=>$left,$this->right=>$right),"$this->id=$parent_id");
        // return the right value of this node + 1
        return $right + 1;
	}
	
	public function beforeSave($event) {
		$owner=$this->getOwner();
		
		if ($owner->getIsNewRecord()) {//如果是添加
			$parent_id=$owner->getAttribute($this->parent_id);
			if ($parent_id) {
				$parent=$owner->find("$this->id = '$parent_id'");
				if ($parent===null) {
					$owner->addError($this->parent_id,'您所选择的父节点不存在');
					$event->isValid=false;
				} else {
					// 根据父节点的左值和右值更新数据
					$owner->updateAll(array($this->left=>new CDbExpression("$this->left + 2")),"$this->left >= '{$parent->getAttribute($this->right)}'");
					$owner->updateAll(array($this->right=>new CDbExpression("$this->right + 2")),"$this->right >= '{$parent->getAttribute($this->right)}'");
					//
					$owner->setAttribute($this->level,$parent->getAttribute($this->level)+1);
					$owner->setAttribute($this->left,$parent->getAttribute($this->right));
					$owner->setAttribute($this->right,$parent->getAttribute($this->right)+1);
				}
			} else {
				if ($this->isAllowedManyRoots===false && $owner->count("$this->parent_id = 0")) {
					$owner->addError($this->parent_id,'根节点只允许一个');
					$event->isValid=false;
				} else {
					$tableName=$owner->tableName();
					$maxRight=$owner->getDbConnection()->createCommand("SELECT MAX($this->right) FROM $tableName")->queryScalar();
					
					$owner->setAttribute($this->level,0);
					$owner->setAttribute($this->left,$maxRight+1);
					$owner->setAttribute($this->right,$maxRight+2);
				}
			}
		} else {//如果处于编辑状态
			//获取未编辑前的数据
			$old=$owner->find("$this->id = '".$owner->getAttribute($this->id)."'");
			if ($owner->getAttribute($this->parent_id) != $owner->getAttribute($this->id) && $owner->getAttribute($this->parent_id) != $old->getAttribute($this->parent_id)) {//改变父节点时
				if ($this->isAllowedManyRoots===false && $old->getAttribute($this->parent_id) && !$owner->getAttribute($this->parent_id)) {
					$owner->addError($this->parent_id,'根节点只允许一个');
					$event->isValid=false;
				} else {
					$parent=$owner->find("$this->id = '".$owner->getAttribute($this->parent_id)."'");
					if ($parent===null) {
						$owner->addError($this->parent_id,'您所选择的父节点不存在');
						$event->isValid=false;
					} elseif ($owner->getAttribute($this->left)<$parent->getAttribute($this->left) && $owner->getAttribute($this->right)>$parent->getAttribute($this->right)) {
						$owner->addError($this->parent_id,'不允许的父节点，因为所选父节点在当前子节点里');
						$event->isValid=false;
					} else {
						//原来的父节点
						$oldParent=$owner->find("$this->id = '".$old->getAttribute($this->parent_id)."'");
						//level的偏移量
						$lvl=$parent->getAttribute($this->level)-$oldParent->getAttribute($this->level);
						//当前节点的所有子节点(包含当前节点);
						$nodeChildren=$owner->findAll("$this->left>={$owner->getAttribute($this->left)} AND $this->right<={$owner->getAttribute($this->right)}");
						$nodeChildrenIds=array();
						foreach ($nodeChildren as $row) {
							$nodeChildrenIds[]=$row->id;
						}
						$nodeChildrenIds=implode(',',$nodeChildrenIds);
						$value=$owner->getAttribute($this->right)-$owner->getAttribute($this->left);
						if ($parent->getAttribute($this->right)>$owner->getAttribute($this->right)) {
							$owner->updateAll(array($this->left=>new CDbExpression("$this->left-$value-1")),"$this->left>{$owner->getAttribute($this->right)} AND rgt<={$parent->getAttribute($this->right)}");
							$owner->updateAll(array($this->right=>new CDbExpression("$this->right-$value-1")),"$this->right>{$owner->getAttribute($this->right)} AND rgt<{$parent->getAttribute($this->right)}");
							$value=$parent->getAttribute($this->right)-$owner->getAttribute($this->right)-1;
							$owner->updateAll(array($this->left=>new CDbExpression("$this->left+$value"),$this->right=>new CDbExpression("$this->right+$value"),$this->level=>new CDbExpression("$this->level+$lvl")),"id IN ($nodeChildrenIds)");
						} else {
							//更新左值
							$owner->updateAll(array($this->left=>new CDbExpression("$this->left+$value+1")),"$this->left>{$parent->getAttribute($this->right)} AND $this->left<{$owner->getAttribute($this->left)}");
							//更新右值
							$owner->updateAll(array($this->right=>new CDbExpression("$this->right+$value+1")),"$this->right>={$parent->getAttribute($this->right)} AND $this->right<{$owner->getAttribute($this->left)}");
							//更新自己以及子类
							$value=$node->getAttribute($this->left)-$parent->getAttribute($this->right);
							$owner->updateAll(array($this->left=>new CDbExpression("$this->left-$value"),$this->right=>new CDbExpression("$this->right-$value"),$this->level=>new CDbExpression("$this->level+$lvl")),"id IN ($nodeChildrenIds)");
						}
					}
				}
			}
		}
	}
	
	public function afterDelete($event) {
		$owner=$this->getOwner();
		
		$left=$owner->getAttribute($this->left);
		$right=$owner->getAttribute($this->right);
		$span=$right-$left+1;
		$owner->deleteAll("$this->left > '$left' AND $this->right < '$right'");
        $owner->updateAll(array($this->left=>new CDbExpression("$this->left - {$span}")),"$this->left > $right");
        $owner->updateAll(array($this->right=>new CDbExpression("$this->right - {$span}")),"$this->right > $right");
	}
	
	//根据主键获取当前节点
	public function node($id=null) {
		$owner=$this->getOwner();
		if ($id===null) {
			$condition="$this->parent_id = 0";
		} else {
			$condition="$this->id = '$id'";
		}
		return $owner->find($condition);
	}
	
	//获取所有根节点
	//$force是否强制显示根,
	public function roots($force=false) {
		$owner=$this->getOwner();
		
		$criteria=new CDbCriteria;
		$criteria->order=$this->left." ASC";
		$criteria->condition="$this->parent_id = 0";
		if ($force===true || $this->isAllowedManyRoots===true) {
			return $owner->findAll($criteria);
		} else {
			$root=$owner->find($criteria);
			if ($root) {
				$criteria->condition="$this->parent_id = '{$root->getAttribute($this->id)}'";
				return $owner->findAll($criteria);
			} else {
				return array();
			}
		}
	}
	
	public function items() {
		$owner=$this->getOwner();
		$rows=$owner->findAll(array('order'=>"$this->left ASC"));
    	$listData=array();
    	foreach ($rows as $row) {
    		$listData[$row->getAttribute($this->id)]='|'.str_repeat('-',$row->getAttribute($this->level)).$row->getAttribute($this->name);
    	}
        return $listData;
	}
	
	//获取从该节点到根节点的路径
	public function getPath($showRoot=true) {
		$owner=$this->getOwner();
		
		$criteria=new CDbCriteria;
		$criteria->order=$this->left." ASC";
		$criteria->condition="$this->left < '{$owner->getAttribute($this->left)}' AND $this->right > '{$owner->getAttribute($this->right)}'";
		if ($showRoot===false) {
			$criteria->addCondition("$this->parent_id <> 0");
		}
		return $owner->findAll($criteria);
	}
	
	//获取父节点
	public function getParent() {
		$owner=$this->getOwner();
		return $owner->find("$this->id = {$owner->getAttribute($this->parent_id)}");
	}
	
	public function getPrev() {
		$owner=$this->getOwner();
		return $owner->find("$this->right = ".($owner->getAttribute($this->left)-1));
	}
	
	public function getNext() {
		$owner=$this->getOwner();
		return $owner->find("$this->left = ".($owner->getAttribute($this->right)+1));
	}
	
	
	
	//获取子节点
	public function getChildren($direct = true) {
		$owner=$this->getOwner();
		
		$criteria=new CDbCriteria;
		$criteria->order=$this->left." ASC";
		
		if ($direct===true) {
			$criteria->condition="$this->parent_id = '{$owner->getAttribute($this->id)}'";
		} else {
			$criteria->condition="$this->left < '{$owner->getAttribute($this->left)}' AND $this->right > '{$owner->getAttribute($this->right)}'";
		}
		
		return $owner->findAll($criteria);
	}
	
	public function getChildrenIds() {
		$owner=$this->getOwner();
		
		$tableName=$owner->tableName();
		return $owner->getDbConnection()->createCommand("SELECT id FROM $tableName WHERE $this->left >= '{$owner->getAttribute($this->left)}' AND $this->right <= '{$owner->getAttribute($this->right)}'")->queryColumn();
	}
	
	public function getHasChildren() {
		$owner=$this->getOwner();
		return $owner->getAttribute($this->right)-$owner->getAttribute($this->left)>1;
	}
	
	public function moveUp() {
		$owner=$this->getOwner();
		
		if ($owner->getAttribute($this->parent_id)) {
			$parent=$owner->find("$this->id = '{$owner->getAttribute($this->parent_id)}'");
			if ($parent===null) {
				$owner->addError($this->parent_id,'找不到父类，可能该树已损坏');
				return false;
			} 
			if ($owner->getAttribute($this->left)-$parent->getAttribute($this->left)<=1) {
				$owner->addError($this->parent_id,'已经置顶，无法再往上移');
				return false;
			}
		} elseif ($this->isAllowedManyRoots===false) {
			$owner->addError($this->parent_id,'根节点无法移动');
			return false;
		}
		
		$prev=$this->getPrev();
		if ($prev===null) {
			$owner->addError($this->parent_id,'前面已经没有节点，无法再往上移');
			return false;
		} else {
			$tableName=$owner->tableName();
			$maxRight=$owner->getDbConnection()->createCommand("SELECT MAX($this->right) FROM $tableName")->queryScalar();
			$shift=$maxRight-$prev->getAttribute($this->left)+1;
			$owner->updateAll(array($this->right=>new CDbExpression("$this->right + $shift")),"$this->right BETWEEN {$prev->getAttribute($this->left)} AND {$prev->getAttribute($this->right)}");
			$owner->updateAll(array($this->left=>new CDbExpression("$this->left + $shift")),"$this->left BETWEEN {$prev->getAttribute($this->left)} AND {$prev->getAttribute($this->right)}");
			
			$shift=$owner->getAttribute($this->left)-$prev->getAttribute($this->left);
			$owner->updateAll(array($this->right=>new CDbExpression("$this->right - $shift")),"$this->right BETWEEN {$owner->getAttribute($this->left)} AND {$owner->getAttribute($this->right)}");
			$owner->updateAll(array($this->left=>new CDbExpression("$this->left - $shift")),"$this->left BETWEEN {$owner->getAttribute($this->left)} AND {$owner->getAttribute($this->right)}");
			
			$shift=$maxRight-$prev->getAttribute($this->left)-($owner->getAttribute($this->right)-$owner->getAttribute($this->left));
			$owner->updateAll(array($this->right=>new CDbExpression("$this->right - $shift")),"$this->right > $maxRight");
			$owner->updateAll(array($this->left=>new CDbExpression("$this->left - $shift")),"$this->left > $maxRight");
		}
		return true;
	}
	
	
	public function moveDown() {
		$owner=$this->getOwner();
		
		if ($owner->getAttribute($this->parent_id)) {
			$parent=$owner->find("$this->id = '{$owner->getAttribute($this->parent_id)}'");
			if ($parent===null) {
				$owner->addError($this->parent_id,'找不到父类，可能该树已损坏');
				return false;
			} 
			if ($parent->getAttribute($this->right)-$owner->getAttribute($this->right)<=1) {
				$owner->addError($this->parent_id,'已经置底，无法再往下移');
				return false;
			}
		} elseif ($this->isAllowedManyRoots===false) {
			$owner->addError($this->parent_id,'根节点无法移动');
			return false;
		}
		
		$next=$this->getNext();
		if ($next===null) {
			$owner->addError($this->parent_id,'后面已经没有节点，无法再往下移');
			return false;
		} else {
			$tableName=$owner->tableName();
			$maxRight=$owner->getDbConnection()->createCommand("SELECT MAX($this->right) FROM $tableName")->queryScalar();
			$shift=$maxRight-$owner->getAttribute($this->left)+1;
			$owner->updateAll(array($this->right=>new CDbExpression("$this->right + $shift")),"$this->right BETWEEN {$owner->getAttribute($this->left)} AND {$owner->getAttribute($this->right)}");
			$owner->updateAll(array($this->left=>new CDbExpression("$this->left + $shift")),"$this->left BETWEEN {$owner->getAttribute($this->left)} AND {$owner->getAttribute($this->right)}");
			
			$shift=$next->getAttribute($this->left)-$owner->getAttribute($this->left);
			$owner->updateAll(array($this->right=>new CDbExpression("$this->right - $shift")),"$this->right BETWEEN {$next->getAttribute($this->left)} AND {$next->getAttribute($this->right)}");
			$owner->updateAll(array($this->left=>new CDbExpression("$this->left - $shift")),"$this->left BETWEEN {$next->getAttribute($this->left)} AND {$next->getAttribute($this->right)}");
			
			$shift=$maxRight-$owner->getAttribute($this->left)-($next->getAttribute($this->right)-$next->getAttribute($this->left));
			$owner->updateAll(array($this->right=>new CDbExpression("$this->right - $shift")),"$this->right > $maxRight");
			$owner->updateAll(array($this->left=>new CDbExpression("$this->left - $shift")),"$this->left > $maxRight");
		}
		return true;
	}

	
}